Package: rtracklayer
Title: R interface to genome annotation files and the UCSC genome
        browser
Version: 1.64.0
Author: Michael Lawrence, Vince Carey, Robert Gentleman
Depends: R (>= 3.5.0), methods, GenomicRanges (>= 1.37.2)
Imports: XML (>= 1.98-0), BiocGenerics (>= 0.35.3), S4Vectors (>=
        0.23.18), IRanges (>= 2.13.13), XVector (>= 0.19.7),
        GenomeInfoDb (>= 1.15.2), Biostrings (>= 2.47.6), zlibbioc,
        curl, httr, Rsamtools (>= 1.31.2), GenomicAlignments (>=
        1.15.6), BiocIO, tools, restfulr (>= 0.0.13)
Suggests: BSgenome (>= 1.33.4), humanStemCell, microRNA (>= 1.1.1),
        genefilter, limma, org.Hs.eg.db, hgu133plus2.db,
        GenomicFeatures, BSgenome.Hsapiens.UCSC.hg19,
        TxDb.Hsapiens.UCSC.hg19.knownGene, RUnit
LinkingTo: S4Vectors, IRanges, XVector
Description: Extensible framework for interacting with multiple genome 
  browsers (currently UCSC built-in) and manipulating 
  annotation tracks in various formats (currently GFF, BED, bedGraph, BED15,
  WIG, BigWig and 2bit built-in). The user may export/import tracks to/from the
  supported browsers, as well as query and modify the browser state, such as the
  current viewport.
Maintainer: Michael Lawrence <michafla@gene.com>
License: Artistic-2.0 + file LICENSE
Collate: io.R web.R ranges.R trackDb.R browser.R ucsc.R readGFF.R gff.R
        bed.R wig.R utils.R bigWig.R bigBed.R chain.R quickload.R
        trackhub.R twobit.R fasta.R tabix.R bam.R trackTable.R index.R
        test_rtracklayer_package.R ncbi.R igv.R zzz.R
biocViews: Annotation,Visualization,DataImport
git_url: https://git.bioconductor.org/packages/rtracklayer
git_branch: RELEASE_3_19
git_last_commit: ba889ee
git_last_commit_date: 2024-04-30
Repository: Bioconductor 3.19
Date/Publication: 2024-04-30
NeedsCompilation: yes
Packaged: 2024-05-01 05:46:41 UTC; biocbuild
Built: R 4.4.0; aarch64-apple-darwin23.2.0; 2024-05-09 11:41:11 UTC; unix
